#include "EMDMA.h"

#include "stdint.h"
#include <sys/platform.h>

#pragma align 32
#pragma section("seg_l1_block1")
uint32_t ucIntBuff1[MAX_DMA_SIZE_BYTES];

#pragma align 32
#pragma section("seg_l1_block2")
uint32_t ucIntBuff2[MAX_DMA_SIZE_BYTES];
uint32_t uiClockStart;
uint32_t uiClockStop;
uint32_t uiCycles;

float EMDMA_Throughput_Measure(EMDMA_PARAM_LIST EMDMAParam)
{
	float fThroughput;
	uint32_t  pint;
	uint32_t  pext;
	uint32_t uiDMACTL=BITM_EMDMA_CTL_EN;
	uint32_t uisize;

	/*Configuring EMDMAs as secure masters*/
	*pREG_SPU0_SECUREP112=0x3;

	pint=GetMemPointer(EMDMAParam.uiIntMem);
	pext=GetMemPointer(EMDMAParam.uiExtMem);

	uisize=EMDMAParam.uiBytes/4;

	if(EMDMAParam.uiDirection==EMDMA_WRITE)
		uiDMACTL|=BITM_EMDMA_CTL_TRAN;

	uiClockStart = clock();

	if(EMDMAParam.uiEMDMAStreamNo==0)
	{
		EMDMA_CONFIG(0, uiDMACTL, pint, 1, uisize, pext, 1, uisize);
		WAIT_FOR_EMDMA_OVER(0);
		DISABLE_EMDMA(0);
	}

	else if(EMDMAParam.uiEMDMAStreamNo==1)
	{
		EMDMA_CONFIG(1, uiDMACTL, pint, 1, uisize, pext, 1, uisize);
		WAIT_FOR_EMDMA_OVER(1);
		DISABLE_EMDMA(1);
	}

	uiClockStop = clock();

	uiCycles=uiClockStop-uiClockStart;

	fThroughput=((float)(EMDMAParam.uiBytes)*EMDMAParam.fCoreClockSpeedMHz)/((float)uiCycles);

	return fThroughput;
}

uint32_t GetMemPointer(uint32_t uiMemType)
{
	if(uiMemType==MEMTYPE_L1S1)
		return ((uint32_t)(&ucIntBuff1)>>2)|0xA000000;
	else if(uiMemType==MEMTYPE_L1S2)
		return ((uint32_t)(&ucIntBuff2)>>2)|0xA100000;
	else if(uiMemType==MEMTYPE_L1S1_CORE2)
			return ((uint32_t)(&ucIntBuff2)>>2)|0xA200000;
	else if(uiMemType==MEMTYPE_L1S2_CORE2)
			return ((uint32_t)(&ucIntBuff2)>>2)|0xA300000;
	else if(uiMemType==MEMTYPE_L2)
			return (L2_START_ADDRESS>>2);
	else if(uiMemType==MEMTYPE_L3_DMC0)
			return (DMC0_START_ADDRESS>>2);
	else if(uiMemType==MEMTYPE_L3_DMC1)
			return (DMC1_START_ADDRESS>>2);
	else
		return ((uint32_t)(&ucIntBuff1)>>2)|0xA000000;
}

