/*The example code involves the following conditions.
	CCLK = 450 MHz, DCLK = 450 MHz, SYSCLK = 225MHz, SCLK0 = SCLK1 = 112.5 MHz
	- EPPI0(SCB4): 24-bit transmit, internal clock and frame sync mode at EPPICLK = 56.25 MHz. Required throughput = 56.25 MHz * 3 = 168.75 MB/s.
	- MDMA0 channel 8, required throughput =  112.5 MHz * 4 <= 450 MB/s.
	- MDMA1 channel 18, required throughput = 112.5 MHz * 4 <= 450 MB/s.
	- MDMA2 channel 39, required throughput = 225   MHz * 4 <= 900 MB/s.
	- MDMA3 channel 43, required throughput = 225   MHz * 8 <= 1800 MB/s.
	- Throughout requirements depend upon the corresponding DMAx_BWLCNT register values. If not programmed, the MDMA channels request for the bandwidth with full throttle.

The code can be run under different conditions by commenting/uncommenting the corresponding conditions/optimization techniques in the "main.h" file.
*/

#include "main.h"

volatile clock_t clock_start_mdma0=0;
volatile clock_t clock_stop_mdma0=0;
volatile clock_t cycles_mdma0=0;
volatile clock_t cycles_mdma0_pre;
float throughput_mdma0=0;

volatile clock_t clock_start_mdma1=0;
volatile clock_t clock_stop_mdma1=0;
volatile clock_t cycles_mdma1=0;
volatile clock_t cycles_mdma1_pre;
float throughput_mdma1=0;

volatile clock_t clock_start_mdma2=0;
volatile clock_t clock_stop_mdma2=0;
volatile clock_t cycles_mdma2=0;
volatile clock_t cycles_mdma2_pre;
float throughput_mdma2=0;

volatile clock_t clock_start_mdma3;
volatile clock_t clock_stop_mdma3=0;
volatile clock_t cycles_mdma3=0;
volatile clock_t cycles_mdma3_pre;
float throughput_mdma3=0;

volatile clock_t clock_start_ppi0;
volatile clock_t clock_stop_ppi0=0;
volatile clock_t cycles_ppi0;
volatile clock_t cycles_ppi0_pre;
float throughput_ppi0=0;

float throughput_total=0;


#pragma align 32
unsigned char ucIntDest2Buff[MAX_SIZE];

#pragma align 32
unsigned char ucIntDest1Buff[MAX_SIZE];

#pragma align 32
unsigned char ucIntDestBuff[MAX_SIZE];

#pragma align 32
unsigned char ucIntSrc1Buff[MAX_SIZE];

void CGUInit(void);
void mdma0_Interrupt_Handler(uint32_t iid, void* handlerArg);
void mdma1_Interrupt_Handler(uint32_t iid, void* handlerArg);
void mdma2_Interrupt_Handler(uint32_t iid, void* handlerArg);
void mdma3_Interrupt_Handler(uint32_t iid, void* handlerArg);


int DMA_Count_PPI0=0;
int DMA_Count_mdma0=0;
int DMA_Count_mdma1=0;
int DMA_Count_mdma2=0;
int DMA_Count_mdma3=0;

int Error_Count_PPI0=0;

CGU_Settings CGU0_PARAM_LIST_DEFAULT;
DMC_PARAM DMC_Param_List;

int main(void)
{
	//Initialize the DMC parameters list
	DMC_Param_List.dmc_no=0;
	DMC_Param_List.ddr_mode=DDR3_MODE;
	DMC_Param_List.phy_init_required=true;
	DMC_Param_List.cclk_dclk_ratio=1;
	DMC_Param_List.anomaly_20000037_applicable=true;
	DMC_Param_List.dmc_ctl_value=DMC_CTL_VALUE;
	DMC_Param_List.dmc_cfg_value=DMC_CFG_VALUE;
	DMC_Param_List.dmc_tr0_value=DMC_TR0_VALUE;
	DMC_Param_List.dmc_tr1_value=DMC_TR1_VALUE;
	DMC_Param_List.dmc_tr2_value=DMC_TR2_VALUE;
	DMC_Param_List.dmc_mr0_value=DMC_MR0_VALUE;
	DMC_Param_List.dmc_mr1_value=DMC_MR1_VALUE;
	DMC_Param_List.dmc_mr2_value=DMC_MR2_VALUE;
	DMC_Param_List.padctl2_value=DMC_PADCTL2_VALUE;
	DMC_Param_List.dmc_cphyctl_value=DMC_CPHYCTL_VALUE;
	DMC_Param_List.dmc_data_calib_add_value=DMC0_DATA_CALIB_ADD;

	//Initialize CGU - PLLCLK = 450 MHz, CCLK = 450 MHz, SYCLK = 225 MHz, DCLK = 450 MHz, SCLK0 = SCLK1 = 112.5 MHz
	CGUInit();

	//Initialize DMC0
	DMCInit(DMC_Param_List);

	//Configuring sync mode
	*pREG_SCB1_MST00_SYNC=2;

	unsigned int uiBytes=0; int i;

	//Enable the interrupts globally
	*pREG_SEC0_GCTL=ENUM_SEC_GCTL_EN;
	*pREG_SEC0_CCTL1=ENUM_SEC_CCTL_EN;

	//Enable mdma2/2 streams as secure master
	*pREG_SPU0_SECUREP53=0x3;
	*pREG_SPU0_SECUREP54=0x3;
	*pREG_SPU0_SECUREP55=0x3;
	*pREG_SPU0_SECUREP56=0x3;

	//Work unit size = 4 pages = 8K words
	uiBytes=2048*4;

	//Configuring sync mode
	*pREG_SCB1_MST00_SYNC=2;

	*pREG_SPU0_SECUREP92=0x3;
	*pREG_SPU0_SECUREP93=0x3;
	*pREG_SPU0_SECUREP68=0x3;
	*pREG_SPU0_SECUREP102=0x3;
	*pREG_SPU0_SECUREP103=0x3;
	*pREG_SPU0_SECUREP104=0x3;
	*pREG_SPU0_SECUREP105=0x3;

	//Enables PPI0 DMA
	#ifdef ENABLE_PPI0_DMA_168_75MBPS
		adi_int_InstallHandler(INTR_EPPI0_CH0_DMA, EPPI0_Interrupt_Handler, NULL, true);
		PDMA_CONFIG(28,PPI0_DMA_CFG, PPI0_DMA_DMC_START_ADDRESS, PPI0_MSIZE, uiBytes/PPI0_MSIZE);
		for(i=0;i<1000;i++);
		EPPI_init(0,uiBytes,EPPI0_CTL_CFG, EPPI0_CTL2_CFG );
	#endif

	//Enables mdma1
	#ifdef ENABLE_MDMA0_DMA_450MBPS
		adi_int_InstallHandler(INTR_MDMA0_DST, mdma0_Interrupt_Handler, NULL, true);
		MDMA_CONFIG(8, SOURCE_DMA_CFG_MDMA0 , MDMA0_DMA_DMC_START_ADDRESS, SOURCE_MSIZE_MDMA0, uiBytes/SOURCE_MSIZE_MDMA0,9,DEST_DMA_CFG_MDMA0, (uint32_t)ucIntDest2Buff|0x28000000, DEST_MSIZE_MDMA0, uiBytes/DEST_MSIZE_MDMA0);

		#ifdef LIMIT_MDMA0_BW
			*pREG_DMA8_BWLCNT=SOURCE_MSIZE_MDMA0*SYSCLK_SPEED/LIMIT_MDMA0_BW;
		#endif

	#endif

	//Enables mdma1
	#ifdef ENABLE_MDMA1_DMA_450MBPS
		adi_int_InstallHandler(INTR_MDMA1_DST, mdma1_Interrupt_Handler, NULL, true);
		MDMA_CONFIG(18, SOURCE_DMA_CFG_MDMA1 , MDMA1_DMA_DMC_START_ADDRESS, SOURCE_MSIZE_MDMA1, uiBytes/SOURCE_MSIZE_MDMA1,19,DEST_DMA_CFG_MDMA1, (uint32_t)ucIntDest2Buff|0x28000000, DEST_MSIZE_MDMA2, uiBytes/DEST_MSIZE_MDMA2);

		#ifdef LIMIT_MDMA1_BW
			*pREG_DMA18_BWLCNT=SOURCE_MSIZE_MDMA1*SYSCLK_SPEED/LIMIT_MDMA1_BW;//2048;
		#endif

	#endif


	//Enables mdma2
	#ifdef ENABLE_MDMA2_DMA_900MBPS
		adi_int_InstallHandler(INTR_MDMA2_DST, mdma2_Interrupt_Handler, NULL, true);
		MDMA_CONFIG(39, SOURCE_DMA_CFG_MDMA2 , MDMA2_DMA_DMC_START_ADDRESS, SOURCE_MSIZE_MDMA2, uiBytes/SOURCE_MSIZE_MDMA2,40,DEST_DMA_CFG_MDMA2, (uint32_t)ucIntDestBuff|0x28000000, DEST_MSIZE_MDMA2, uiBytes/DEST_MSIZE_MDMA2);

		#ifdef LIMIT_MDMA2_BW
			*pREG_DMA39_BWLCNT=SOURCE_MSIZE_MDMA2*SYSCLK_SPEED/LIMIT_MDMA2_BW;//2048;
		#endif
	#endif

	//Enables mdma3
	#ifdef ENABLE_MDMA3_DMA_1800MBPS
		adi_int_InstallHandler(INTR_MDMA3_DST, mdma3_Interrupt_Handler, NULL, true);
		MDMA_CONFIG(43, SOURCE_DMA_CFG_MDMA3 , MDMA3_DMA_DMC_START_ADDRESS, SOURCE_MSIZE_MDMA3, uiBytes/SOURCE_MSIZE_MDMA3,44,DEST_DMA_CFG_MDMA3, (uint32_t)ucIntDest1Buff|0x28000000, DEST_MSIZE_MDMA3, uiBytes/DEST_MSIZE_MDMA3);

		#ifdef LIMIT_MDMA3_BW
			*pREG_DMA43_BWLCNT=SOURCE_MSIZE_MDMA3*SYSCLK_SPEED/LIMIT_MDMA3_BW;//2048;
		#endif

	#endif

	//Wait for all the DMAs to complete at least 500 times
	while(DMA_Count_PPI0<500||DMA_Count_mdma0<500||DMA_Count_mdma1<500||DMA_Count_mdma2<500||DMA_Count_mdma3<500);


	//Check for any PPI underflow errors
	if(*pREG_EPPI0_STAT&ENUM_EPPI_STAT_YFIFOERR)
			Error_Count_PPI0=1;

	//Disable the peripherals and MDMAs
	*pREG_EPPI0_CTL =0;
	DISABLE_MDMAs(43,44);
	DISABLE_MDMAs(8,9);
	DISABLE_MDMAs(18,19);
	DISABLE_MDMAs(39,40);

	//Print the throughput numbers
	#ifdef ENABLE_PPI0_DMA_168_75MBPS
		if(Error_Count_PPI0)
			printf("FIFO Underflow Error in PPI0 !!!\n\n");
		throughput_ppi0=(float)uiBytes*CCLK_SPEED/cycles_ppi0;
			printf("Throughput PPI0=%f MB/s\n\n", throughput_ppi0);
	#endif

	#ifdef ENABLE_MDMA0_DMA_450MBPS
		throughput_mdma0=(float)uiBytes*CCLK_SPEED/cycles_mdma0;
		printf("Throughput mdma0=%f MB/s\n\n", throughput_mdma0);
	#endif

	#ifdef ENABLE_MDMA1_DMA_450MBPS
		throughput_mdma1=(float)uiBytes*CCLK_SPEED/cycles_mdma1;
		printf("Throughput mdma1=%f MB/s\n\n", throughput_mdma1);
	#endif

	#ifdef ENABLE_MDMA2_DMA_900MBPS
		throughput_mdma2=(float)uiBytes*CCLK_SPEED/cycles_mdma2;
		printf("Throughput mdma2=%f MB/s\n\n", throughput_mdma2);
	#endif

	#ifdef ENABLE_MDMA3_DMA_1800MBPS
		throughput_mdma3=(float)uiBytes*CCLK_SPEED/cycles_mdma3;
		printf("Throughput mdma3=%f MB/s\n\n", throughput_mdma3);
	#endif

	throughput_total = throughput_ppi0 + throughput_mdma0 + throughput_mdma1+ throughput_mdma2 + throughput_mdma3;

	printf("Throughput total=%f MB/s\n\n", throughput_total);

	return 0;
}

void CGUInit(void)
{
	int i;

	//PLLCLK = 450 MHz, CCLK = 450 MHz, SYCLK = 225 MHz, DCLK = 450 MHz, SCLK0 = SCLK1 = 112.5 MHz
	CGU0_PARAM_LIST_DEFAULT.ctl_MSEL=18;
	CGU0_PARAM_LIST_DEFAULT.ctl_DF=0;
	CGU0_PARAM_LIST_DEFAULT.div_CSEL=1;
	CGU0_PARAM_LIST_DEFAULT.div_SYSSEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S0SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_S1SEL=2;
	CGU0_PARAM_LIST_DEFAULT.div_DSEL=1;

	//Set DMCx_PHY_CTL0.RESETDLL bit.
	*pREG_DMC0_PHY_CTL0=0x800;

	//Initialize CGU to change the DCLK frequency
	CGU_Init(0, 0, &CGU0_PARAM_LIST_DEFAULT,  0);

	//Clear DMCx_PHY_CTL0.RESETDLL bit
	*pREG_DMC0_PHY_CTL0=0x000;

	//Wait for 9000 DCLK cycles for the DLL to lock
	for(i=0;i<9000*DMC_Param_List.cclk_dclk_ratio;i++);
}

void EPPI_init(unsigned int uiEppiID, unsigned int uiBytes, unsigned int EPPI_CTL_CFG, unsigned int EPPI_CTL2_CFG)
{
	//EPPI_Pin_Config(uiEppiID, 1, DATAWIDTH);									// Configure the pin multiplexing registers
	EPPI_Control_Config(uiEppiID, EPPI_CTL_CFG, EPPI_CTL2_CFG, 1);						// Configure EPPI control, control2 and clock divisor registers respectively.
	EPPI_Line_Config(uiEppiID, uiBytes, uiBytes, 0);								// Configure EPPI Line, Hcount and Hdelay registers respectively.
	EPPI_FSync_Config(uiEppiID, 3, uiBytes, 0 , 0 );								// Configure EPPI FS1-width, FS1-period, FS2-width and FS2-period registers respectively.
	EPPI_IMASK_Config(uiEppiID, 0);												// Configure EPPI IMASK register.

}



void EPPI0_Interrupt_Handler (uint32_t iid, void* handlerArg)
{
	clock_start_ppi0=clock_stop_ppi0;
	clock_stop_ppi0 = clock();
	cycles_ppi0_pre=cycles_ppi0;

	if(DMA_Count_PPI0>0)
	{
		cycles_ppi0=clock_stop_ppi0-clock_start_ppi0;
		cycles_ppi0=(DMA_Count_PPI0*cycles_ppi0_pre+cycles_ppi0)/(DMA_Count_PPI0+1);
	}

	*pREG_DMA28_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit
	DMA_Count_PPI0++;
	if(DMA_Count_PPI0==1)
		*pREG_DMA28_STAT |= ENUM_EPPI_STAT_YFIFOERR;			 //Clear the W1C interrupt status bit

}


void mdma0_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	clock_start_mdma0=clock_stop_mdma0;
	clock_stop_mdma0 = clock();
	cycles_mdma0_pre=cycles_mdma0;
	cycles_mdma0=clock_stop_mdma0-clock_start_mdma0;
	if(DMA_Count_mdma0>1)
	{
		cycles_mdma0=(DMA_Count_mdma0*cycles_mdma0_pre+cycles_mdma0)/(DMA_Count_mdma0+1);
	}

	*pREG_DMA9_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Count_mdma0++;
}

void mdma1_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	clock_start_mdma1=clock_stop_mdma1;
	clock_stop_mdma1 = clock();
	cycles_mdma1_pre=cycles_mdma1;
	cycles_mdma1=clock_stop_mdma1-clock_start_mdma1;
	if(DMA_Count_mdma1>1)
	{
		cycles_mdma1=(DMA_Count_mdma1*cycles_mdma1_pre+cycles_mdma1)/(DMA_Count_mdma1+1);
	}

	*pREG_DMA19_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Count_mdma1++;
}

void mdma2_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	clock_start_mdma2=clock_stop_mdma2;
	clock_stop_mdma2 = clock();
	cycles_mdma2_pre=cycles_mdma2;
	cycles_mdma2=clock_stop_mdma2-clock_start_mdma2;
	if(DMA_Count_mdma2>1)
	{
		cycles_mdma2=(DMA_Count_mdma2*cycles_mdma2_pre+cycles_mdma2)/(DMA_Count_mdma2+1);
	}

	*pREG_DMA40_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Count_mdma2++;
}

void mdma3_Interrupt_Handler(uint32_t iid, void* handlerArg)
{
	clock_start_mdma3=clock_stop_mdma3;
	clock_stop_mdma3= clock();
	cycles_mdma3_pre=cycles_mdma3;
	cycles_mdma3=clock_stop_mdma3-clock_start_mdma3;
	if(DMA_Count_mdma3>1)
	{
		cycles_mdma3=(DMA_Count_mdma3*cycles_mdma3_pre+cycles_mdma3)/(DMA_Count_mdma3+1);
	}

	*pREG_DMA44_STAT |= ENUM_DMA_STAT_IRQDONE;			 //Clear the W1C interrupt status bit

	DMA_Count_mdma3++;
}

