#include "stdint.h"
#include <sys/platform.h>

enum MEMTYPE
{
	MEMTYPE_L1S1,
	MEMTYPE_L1S2,
	MEMTYPE_L1S1_CORE2,
	MEMTYPE_L1S2_CORE2,
	MEMTYPE_L2,
	MEMTYPE_L3_DMC0,
	MEMTYPE_L3_DMC1
};

typedef struct
{
	uint32_t uiMDMAStreamNo;
	uint32_t uiSrcMem;
	uint32_t uiDestMem;
	uint32_t uiBytes;
	uint32_t uiSrcMSIZE;
	uint32_t uiDstMSIZE;
	float 	 fMDMAClockSpeedMHz;
}MDMA_PARAM_LIST;

//This macro can be used to configure an MDMA stream by providing the source and destination DMA channel details
#define MDMA_CONFIG(S_MDMA_CHANNEL,S_CFG, S_ADDRSTART, S_MOD, S_COUNT,D_MDMA_CHANNEL,D_CFG, D_ADDRSTART, D_MOD, D_COUNT)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = S_CFG;\
		*pREG_DMA##S_MDMA_CHANNEL##_ADDRSTART = S_ADDRSTART;\
		*pREG_DMA##S_MDMA_CHANNEL##_XCNT = S_COUNT;\
		*pREG_DMA##S_MDMA_CHANNEL##_XMOD = S_MOD;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = D_CFG;\
		*pREG_DMA##D_MDMA_CHANNEL##_ADDRSTART = D_ADDRSTART;\
		*pREG_DMA##D_MDMA_CHANNEL##_XCNT = D_COUNT;\
		*pREG_DMA##D_MDMA_CHANNEL##_XMOD = D_MOD;\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG |= 0x1;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG |= 0x1;




//Macro to wait for DMA done
#define WAIT_FOR_DMADONE(DMA_CHANNEL)\
		while(!(*pREG_DMA##DMA_CHANNEL##_STAT & ENUM_DMA_STAT_IRQDONE));

//Macro to disable MDMA channels
#define DISABLE_MDMA(S_MDMA_CHANNEL, D_MDMA_CHANNEL)\
		*pREG_DMA##S_MDMA_CHANNEL##_CFG = 0;\
		*pREG_DMA##D_MDMA_CHANNEL##_CFG = 0;

#define MAX_DMA_SIZE_BYTES 16384
#define L2_START_ADDRESS 0x20080000
#define DMC0_START_ADDRESS 0x80000000
#define DMC1_START_ADDRESS 0xC0000000


float MDMA_Throughput_Measure(MDMA_PARAM_LIST MDMAParam);
uint32_t GetMemPointer(uint32_t uiMemType);
uint32_t GetMSIZEConfig(uint32_t uiMSIZE);
